@@ -1,11 +1,10 @@
-#-*- perl -*-
 
 package MIME::Charset;
 use 5.005;
 
 =head1 NAME
 
-MIME::Charset - Charset Information for MIME
+MIME::Charset - Charset Informations for MIME
 
 =head1 SYNOPSIS
 
@@ -13,10 +12,10 @@ MIME::Charset - Charset Information for MIME
 
     $charset = MIME::Charset->new("euc-jp");
 
-Getting charset information:
+Getting charset informations:
 
     $benc = $charset->body_encoding; # e.g. "Q"
-    $cset = $charset->as_string; # e.g. "US-ASCII"
+    $cset = $charset->canonical_charset; # e.g. "US-ASCII"
     $henc = $charset->header_encoding; # e.g. "S"
     $cset = $charset->output_charset; # e.g. "ISO-2022-JP"
 
@@ -25,30 +24,27 @@ Translating text data:
     ($text, $charset, $encoding) =
         $charset->header_encode(
            "\xc9\xc2\xc5\xaa\xc0\xde\xc3\xef\xc5\xaa".
-           "\xc7\xd1\xca\xaa\xbd\xd0\xce\xcf\xb4\xef",
-           Charset => 'euc-jp');
-    # ...returns e.g. (<converted>, "ISO-2022-JP", "B").
+           "\xc7\xd1\xca\xaa\xbd\xd0\xce\xcf\xb4\xef");
+    # ...returns e.g. (<converted>, "ISO-2022-JP", "B");
 
     ($text, $charset, $encoding) =
         $charset->body_encode(
             "Collectioneur path\xe9tiquement ".
-            "\xe9clectique de d\xe9chets",
-            Charset => 'latin1');
-    # ...returns e.g. (<original>, "ISO-8859-1", "QUOTED-PRINTABLE").
+            "\xe9clectique de d\xe9chets");
+    # ...returns e.g. (<original>, "ISO-8859-1", "QUOTED-PRINTABLE");
 
     $len = $charset->encoded_header_len(
-        "Perl\xe8\xa8\x80\xe8\xaa\x9e",
-        Charset => 'utf-8',
-        Encoding => "b");
-    # ...returns e.g. 28.
+        "Perl\xe8\xa8\x80\xe8\xaa\x9e", "b"); # e.g. 28
 
 Manipulating module defaults:
 
+    use MIME::Charset;
+
     MIME::Charset::alias("csEUCKR", "euc-kr");
     MIME::Charset::default("iso-8859-1");
     MIME::Charset::fallback("us-ascii");
 
-Non-OO functions (may be deprecated in near future):
+Non-OOP functions (may be deprecated in near future):
 
     use MIME::Charset qw(:info);
 
@@ -78,7 +74,7 @@ Non-OO functions (may be deprecated in near future):
 
 =head1 DESCRIPTION
 
-MIME::Charset provides information about character sets used for
+MIME::Charset provides informations about character sets used for
 MIME messages on Internet.
 
 =head2 Definitions
@@ -92,10 +88,12 @@ The B<encoding> is that used in MIME to refer to a method of representing
 a body part or a header body as sequence(s) of printable US-ASCII
 characters.
 
+=over 4
+
 =cut
 
 use strict;
-use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS $Config);
+use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(body_encoding canonical_charset header_encoding output_charset
@@ -109,21 +107,12 @@ use Exporter;
 		);
 use Carp qw(croak);
 
-use constant USE_ENCODE => ($] >= 5.007003)? 'Encode': '';
+use constant USE_ENCODE => ($] >= 5.008001)? 'Encode': '';
 
 my @ENCODE_SUBS = qw(FB_CROAK FB_PERLQQ FB_HTMLCREF FB_XMLCREF
-		     is_utf8 resolve_alias);
+		     from_to is_utf8 resolve_alias);
 if (USE_ENCODE) {
     eval "use ".USE_ENCODE." \@ENCODE_SUBS;";
-    if ($@) { # Perl 5.7.3 + Encode 0.40
-	eval "use ".USE_ENCODE." qw(is_utf8);";
-	require MIME::Charset::_Compat;
-	for my $sub (@ENCODE_SUBS) {
-	    no strict "refs";
-	    *{$sub} = \&{"MIME::Charset::_Compat::$sub"}
-		unless $sub eq 'is_utf8';
-	}
-    }
 } else {
     require MIME::Charset::_Compat;
     for my $sub (@ENCODE_SUBS) {
@@ -132,14 +121,14 @@ if (USE_ENCODE) {
     }
 }
 
-$VERSION = '1.009.3';
+$VERSION = '1.001';
 
 ######## Private Attributes ########
 
 my $DEFAULT_CHARSET = 'US-ASCII';
 my $FALLBACK_CHARSET = 'UTF-8';
 
-# This table was initially borrowed from Python email package.
+# This table was borrwed from Python email package.
 
 my %CHARSETS = (# input		    header enc body enc output conv
 		'ISO-8859-1' =>		['Q',	'Q',	undef],
@@ -156,21 +145,16 @@ my %CHARSETS = (# input		    header enc body enc output conv
 		'ISO-8859-13' =>	['Q',	'Q',	undef],
 		'ISO-8859-14' =>	['Q',	'Q',	undef],
 		'ISO-8859-15' =>	['Q',	'Q',	undef],
-		'ISO-8859-16' =>	['Q',	'Q',	undef],
 		'WINDOWS-1252' =>	['Q',	'Q',	undef],
 		'VISCII' =>		['Q',	'Q',	undef],
 		'US-ASCII' =>		[undef,	undef,	undef],
 		'BIG5' =>		['B',	'B',	undef],
 		'GB2312' =>		['B',	'B',	undef],
-		'HZ-GB-2312' =>		['B',	undef,	undef],
 		'EUC-JP' =>		['B',	undef,	'ISO-2022-JP'],
 		'SHIFT_JIS' =>		['B',	undef,	'ISO-2022-JP'],
 		'ISO-2022-JP' =>	['B',	undef,	undef],
 		'KOI8-R' =>		['B',	'B',	undef],
-		'TIS-620' =>		['B',	'B',	undef], # cf. Mew
-		'UTF-7' =>		['Q',	undef,	undef],
-		'UTF-8' =>		['S',	'S',	undef],
-		'GSM03.38' =>		[undef,	undef,	undef], # not for MIME
+		'UTF-8' =>		['S',	'B',	undef],
 		# We're making this one up to represent raw unencoded 8bit
 		'8BIT' =>		[undef,	'B',	'ISO-8859-1'],
 		);
@@ -180,28 +164,17 @@ my %CHARSETS = (# input		    header enc body enc output conv
 my %CHARSET_ALIASES = (# unpreferred		preferred
 		       "ASCII" =>		"US-ASCII",
 		       "BIG5-ETEN" =>		"BIG5",
-		       "CP1250" =>		"WINDOWS-1250",
 		       "CP1251" =>		"WINDOWS-1251",
 		       "CP1252" =>		"WINDOWS-1252",
-		       "CP1253" =>		"WINDOWS-1253",
-		       "CP1254" =>		"WINDOWS-1254",
-		       "CP1255" =>		"WINDOWS-1255",
-		       "CP1256" =>		"WINDOWS-1256",
-		       "CP1257" =>		"WINDOWS-1257",
-		       "CP1258" =>		"WINDOWS-1258",
-		       "CP874" =>		"WINDOWS-874",
 		       "CP936" =>		"GBK",
 		       "CP949" =>		"KS_C_5601-1987",
 		       "EUC-CN" =>		"GB2312",
-		       "HZ" =>			"HZ-GB-2312", # RFC 1842
 		       "KS_C_5601" =>		"KS_C_5601-1987",
 		       "SHIFTJIS" =>		"SHIFT_JIS",
 		       "SHIFTJISX0213" =>	"SHIFT_JISX0213",
-		       "TIS620" =>		"TIS-620", # IANA MIBenum 2259
-		       "UNICODE-1-1-UTF-7" =>	"UTF-7", # RFC 1642 (obs.)
+		       "UNICODE-1-1-UTF-7" =>	"UTF-7",
 		       "UTF8" =>		"UTF-8",
-		       "UTF-8-STRICT" =>	"UTF-8", # Perl internal use
-		       "GSM0338" =>		"GSM03.38", # not for MIME
+		       "UTF-8-STRICT" =>	"UTF-8",
 		       );
 
 # Some vendors encode characters beyond standardized mappings using extended
@@ -215,28 +188,19 @@ my %ENCODERS = (
 				     ['cp1256'],        # Encode::Byte
 				     # ['cp1006'],      # ditto, for Farsi
 				    ],
-		    'ISO-8859-6-I'=>[['cp1256'], ],     # ditto
 		    'ISO-8859-7' => [['cp1253'], ],     # Encode::Byte
 		    'ISO-8859-8' => [['cp1255'], ],     # Encode::Byte
-		    'ISO-8859-8-I'=>[['cp1255'], ],     # ditto
 		    'ISO-8859-9' => [['cp1254'], ],     # Encode::Byte
 		    'ISO-8859-13'=> [['cp1257'], ],     # Encode::Byte
 		    'GB2312'     => [['cp936'], ],      # Encode::CN
-		    'EUC-JP'     => [
-				     ['eucJP-ascii',	'Encode::EUCJPASCII'],
-				     # ['cp51932',	'Encode::EUCJPMS'],
-				    ],
+		    'EUC-JP'     => [['cp51932',        'Encode::EUCJPMS'], ],
 		    'ISO-2022-JP'=> [
-				     ['x-iso2022jp-ascii',
-				      			'Encode::EUCJPASCII'],
-				     # ['iso-2022-jp-ms','Encode::ISO2022JPMS'],
-				     # ['cp50220',      'Encode::EUCJPMS'],
+				     ['cp50220',        'Encode::EUCJPMS'],
 				     # ['cp50221',      'Encode::EUCJPMS'],
+				     ['iso-2022-jp-ms', 'Encode::ISO2022JPMS'],
 				     ['iso-2022-jp-1'], # Encode::JP (note*)
 				    ],
-		    'SHIFT_JIS'  => [
-				     ['cp932'],		# Encode::JP
-				    ],
+		    'SHIFT_JIS'  => [['cp932'], ],      # Encode::JP (note*)
 		    'EUC-KR'     => [['cp949'], ],      # Encode::KR
 		    'BIG5'       => [
 				     # ['big5plus',     'Encode::HanExtra'],
@@ -244,39 +208,29 @@ my %ENCODERS = (
 				     ['cp950'],         # Encode::TW
 				     # ['big5-1984',    'Encode::HanExtra'], 
 				    ],
-		    'TIS-620'    => [['cp874'], ],      # Encode::Byte
+		    'TIS620'     => [['iso-8859-11'], ], # Encode::Byte; NBSP
 		    'UTF-8'      => [['utf8'], ],       # Special name on Perl
 		},
 		'STANDARD' => {
-		    'ISO-8859-6-E'  => [['iso-8859-6'],],# Encode::Byte
-		    'ISO-8859-6-I'  => [['iso-8859-6'],],# ditto
-		    'ISO-8859-8-E'  => [['iso-8859-8'],],# Encode::Byte
-		    'ISO-8859-8-I'  => [['iso-8859-8'],],# ditto
 		    'GB18030'       => [['gb18030',     'Encode::HanExtra'], ],
 		    'EUC-JISX0213'  => [['euc-jisx0213', 'Encode::JIS2K'], ],
 		    'ISO-2022-JP-3' => [['iso-2022-jp-3', 'Encode::JIS2K'], ],
 		    'SHIFT_JISX0213'=> [['shiftjisx0213', 'Encode::JIS2K'], ],
 		    'EUC-TW'        => [['euc-tw',      'Encode::HanExtra'], ],
-		    'HZ-GB-2312'    => [['hz'], ],	# Encode::CN
-		    'TIS-620'       => [['tis620'], ],  # (note*)
-		    'GSM03.38'      => [['gsm0338'], ],	# Encode::GSM0338
-
-		    # (note*) ISO-8859-11 was not registered by IANA.
-		    # L<Encode> treats it as canonical name of ``tis-?620''.
 		},
 );
 
-# ISO-2022-* escape sequences etc. to detect charset from unencoded data.
-my @ESCAPE_SEQS = ( 
-		# ISO-2022-* sequences
-		   # escape seq, possible charset
+# note*: This encoder is not UCM-based.
+
+# ISO-2022-* escape sequnces to detect charset from unencoded data.
+my @ISO2022_SEQ = (# escape seq	possible charset
 		   # Following sequences are commonly used.
 		   ["\033\$\@",	"ISO-2022-JP"],	# RFC 1468
 		   ["\033\$B",	"ISO-2022-JP"],	# ditto
 		   ["\033(J",	"ISO-2022-JP"],	# ditto
 		   ["\033(I",	"ISO-2022-JP"],	# ditto (nonstandard)
 		   ["\033\$(D",	"ISO-2022-JP"],	# RFC 2237 (note*)
-		   # Following sequences are less commonly used.
+		   # Folloing sequences are less commonly used.
 		   ["\033\$)C",	"ISO-2022-KR"],	# RFC 1557
 		   ["\033\$)A",	"ISO-2022-CN"], # RFC 1922
 		   ["\033\$A",	"ISO-2022-CN"], # ditto (nonstandard)
@@ -284,6 +238,7 @@ my @ESCAPE_SEQS = (
 		   ["\033\$*H",	"ISO-2022-CN"], # ditto
 		   # Other sequences will be used with appropriate charset
 		   # parameters, or hardly used.
+		   );
 
 		   # note*: This RFC defines ISO-2022-JP-1, superset of 
 		   # ISO-2022-JP.  But that charset name is rarely used.
@@ -291,25 +246,9 @@ my @ESCAPE_SEQS = (
 		   # sequence so that comatibility with EUC-JP will be
 		   # guaranteed.
 
-		# Singlebyte 7-bit sequences
-		   # escape seq, possible charset
-		   ["\033e",	"GSM03.38"],	# ESTI GSM 03.38 (note*)
-		   ["\033\012",	"GSM03.38"],	# ditto
-		   ["\033<",	"GSM03.38"],	# ditto
-		   ["\033/",	"GSM03.38"],	# ditto
-		   ["\033>",	"GSM03.38"],	# ditto
-		   ["\033\024",	"GSM03.38"],	# ditto
-		   ["\033(",	"GSM03.38"],	# ditto
-		   ["\033\@",	"GSM03.38"],	# ditto
-		   ["\033)",	"GSM03.38"],	# ditto
-		   ["\033=",	"GSM03.38"],	# ditto
-
-		   # note*: This is not used for MIME message.
-		  );
-
 ######## Public Configuration Attributes ########
 
-$Config = {
+our $Config = {
     Detect7bit =>      'YES',
     Mapping =>         'EXTENDED',
     Replacement =>     'DEFAULT',
@@ -318,20 +257,12 @@ eval { require MIME::Charset::Defaults; };
 
 ######## Private Constants ########
 
-my $NON7BITRE = qr{
+my $NONASCIIRE = qr{
     [^\x01-\x7e]
 }x;
 
-my $NONASCIIRE = qr{
-    [^\x09\x0a\x0d\x20\x21-\x7e]
-}x;
-
 my $ISO2022RE = qr{
-    ISO-2022-.+
-}ix;
-
-my $ASCIITRANSRE = qr{
-    HZ-GB-2312 | UTF-7
+    ^ISO-2022-
 }ix;
 
 
@@ -339,22 +270,20 @@ my $ASCIITRANSRE = qr{
 
 =head2 Constructor
 
-=over
-
 =item $charset = MIME::Charset->new([CHARSET [, OPTS]])
 
 Create charset object.
 
-OPTS may accept following key-value pair.
+OPTS may accept following key-value pairs.
 B<NOTE>:
 When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
-conversion will not be performed.  So this option do not have any effects.
+conversion will not be performed.  So these options do not have any effects.
 
 =over 4
 
 =item Mapping => MAPTYPE
 
-Whether to extend mappings actually used for charset names or not.
+Specify extended mappings actually used for charset names.
 C<"EXTENDED"> uses extended mappings.
 C<"STANDARD"> uses standardized strict mappings.
 Default is C<"EXTENDED">.
@@ -367,22 +296,10 @@ sub new {
     my $class = shift;
     my $charset = shift;
     return bless {}, $class unless $charset;
-    return bless {}, $class if 75 < length $charset; # w/a for CPAN RT #65796.
     my %params = @_;
     my $mapping = uc($params{'Mapping'} || $Config->{Mapping});
 
-    if ($charset =~ /\bhz.?gb.?2312$/i) {
-	# workaround: "HZ-GB-2312" mistakenly treated as "EUC-CN" by Encode
-	# (2.12).
-	$charset = "HZ-GB-2312";
-    } elsif ($charset =~ /\btis-?620$/i) {
-	# workaround: "TIS620" treated as ISO-8859-11 by Encode.
-	# And "TIS-620" not known by some versions of Encode (cf.
-	# CPAN RT #20781).
-	$charset = "TIS-620";
-    } else {
-	$charset = resolve_alias($charset) || $charset
-    }
+    $charset = resolve_alias($charset) || $charset;
     $charset = $CHARSET_ALIASES{uc($charset)} || uc($charset);
     my ($henc, $benc, $outcset);
     my $spec = $CHARSETS{$charset};
@@ -410,18 +327,12 @@ sub new {
     }, $class;
 }
 
-my %encoder_cache = ();
-
 sub _find_encoder($$) {
-    my $charset = uc(shift || "");
+    my $charset = uc(shift);
     return undef unless $charset;
     my $mapping = uc(shift);
     my ($spec, $name, $module, $encoder);
 
-    local($@);
-    $encoder = $encoder_cache{$charset, $mapping};
-    return $encoder if ref $encoder;
-
     foreach my $m (('EXTENDED', 'STANDARD')) {
 	next if $m eq 'EXTENDED' and $mapping ne 'EXTENDED';
 	$spec = $ENCODERS{$m}->{$charset};
@@ -433,20 +344,13 @@ sub _find_encoder($$) {
 		next if $@;
 	    }
 	    $encoder = Encode::find_encoding($name);
-	    last if ref $encoder;
+	    return $encoder if ref $encoder;
 	}
-	last if ref $encoder;
     }
-    $encoder ||= Encode::find_encoding($charset);
-    $encoder_cache{$charset, $mapping} = $encoder if $encoder;
-    return $encoder;
+    return Encode::find_encoding($charset);
 }
 
-=back
-
-=head2 Getting Information of Charsets
-
-=over
+=head2 Getting Informations of Charsets
 
 =item $charset->body_encoding
 
@@ -454,8 +358,7 @@ sub _find_encoder($$) {
 
 Get recommended transfer-encoding of CHARSET for message body.
 
-Returned value will be one of C<"B"> (BASE64), C<"Q"> (QUOTED-PRINTABLE),
-C<"S"> (shorter one of either) or
+Returned value will be one of C<"B"> (BASE64), C<"Q"> (QUOTED-PRINTABLE) or
 C<undef> (might not be transfer-encoded; either 7BIT or 8BIT).  This may
 not be same as encoding for message header.
 
@@ -464,7 +367,7 @@ not be same as encoding for message header.
 sub body_encoding($) {
     my $self = shift;
     return undef unless $self;
-    $self = __PACKAGE__->new($self) unless ref $self;
+    $self = MIME::Charset->new($self) unless ref $self;
     $self->{BodyEncoding};
 }
 
@@ -472,14 +375,14 @@ sub body_encoding($) {
 
 =item canonical_charset CHARSET
 
-Get canonical name for charset.
+Get canonical name for charset CHARSET.
 
 =cut
 
 sub canonical_charset($) {
     my $self = shift;
     return undef unless $self;
-    $self = __PACKAGE__->new($self) unless ref $self;
+    $self = MIME::Charset->new($self) unless ref $self;
     $self->{InputCharset};
 }
 
@@ -488,11 +391,26 @@ sub as_string($) {
     $self->{InputCharset};
 }
 
+=item $charset->decode(STRING [,CHECK])
+
+Decode STRING to Unicode.
+
+B<Note>:
+When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
+this function will die.
+
+=cut
+
+sub decode($$$;) {
+    my $self = shift;
+    my $s = shift;
+    my $check = shift || 0;
+    $self->{Decoder}->decode($s, $check);
+}
+
 =item $charset->decoder
 
-Get L<"Encode::Encoding"> object to decode strings to Unicode by charset.
-If charset is not specified or not known by this module,
-undef will be returned.
+Get L<"Encode::Encoding"> object to decode strings by charset.
 
 =cut
 
@@ -501,41 +419,39 @@ sub decoder($) {
     $self->{Decoder};
 }
 
-=item $charset->dup
+=item $charset->encode(STRING [,CHECK])
 
-Get a copy of charset object.
+Encode STRING (Unicode or non-Unicode) using compatible charset recommended
+to be used for messages on Internet (if this module know it).
+Note that string will be decoded then encoded even if compatible charset
+was equal to original charset.
+
+B<Note>:
+When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
+this function will die.
 
 =cut
 
-sub dup($) {
+sub encode($$$;) {
     my $self = shift;
-    my $obj = __PACKAGE__->new(undef);
-    %{$obj} = %{$self};
-    $obj;
+    my $s = shift;
+    my $check = shift || 0;
+
+    unless (is_utf8($s) or $s =~ /[^\x00-\xFF]/) {
+	$s = $self->{Decoder}->decode($s, ($check & 0x1)? FB_CROAK(): 0);
+    }
+    $self->{Encoder}->encode($s, $check);
 }
 
-=item $charset->encoder([CHARSET])
+=item $charset->encoder
 
 Get L<"Encode::Encoding"> object to encode Unicode string using compatible
 charset recommended to be used for messages on Internet.
 
-If optional CHARSET is specified, replace encoder (and output charset
-name) of $charset object with those of CHARSET, therefore,
-$charset object will be a converter between original charset and
-new CHARSET.
-
 =cut
 
-sub encoder($$;) {
+sub encoder($) {
     my $self = shift;
-    my $charset = shift;
-    if ($charset) {
-	$charset = __PACKAGE__->new($charset) unless ref $charset;
-	$self->{OutputCharset} = $charset->{InputCharset};
-	$self->{Encoder} = $charset->{Decoder};
-	$self->{BodyEncoding} = $charset->{BodyEncoding};
-	$self->{HeaderEncoding} = $charset->{HeaderEncoding};
-    }
     $self->{Encoder};
 }
 
@@ -554,7 +470,7 @@ for message body.
 sub header_encoding($) {
     my $self = shift;
     return undef unless $self;
-    $self = __PACKAGE__->new($self) unless ref $self;
+    $self = MIME::Charset->new($self) unless ref $self;
     $self->{HeaderEncoding};
 }
 
@@ -574,16 +490,12 @@ return the result of L<"canonical_charset">.
 sub output_charset($) {
     my $self = shift;
     return undef unless $self;
-    $self = __PACKAGE__->new($self) unless ref $self;
+    $self = MIME::Charset->new($self) unless ref $self;
     $self->{OutputCharset};
 }
 
-=back
-
 =head2 Translating Text Data
 
-=over
-
 =item $charset->body_encode(STRING [, OPTS])
 
 =item body_encode STRING, CHARSET [, OPTS]
@@ -599,15 +511,15 @@ conversion will not be performed.  So these options do not have any effects.
 
 =over 4
 
+=item Replacement => REPLACEMENT
+
+Specifies error handling scheme.  See L<"ERROR HANDLING">.
+
 =item Detect7bit => YESNO
 
 Try auto-detecting 7-bit charset when CHARSET is not given.
 Default is C<"YES">.
 
-=item Replacement => REPLACEMENT
-
-Specifies error handling scheme.  See L<"Error Handling">.
-
 =back
 
 3-item list of (I<converted string>, I<charset for output>,
@@ -628,27 +540,36 @@ sub body_encode {
 	$text = shift;
     } else {
 	$text = $self;
-	$self = __PACKAGE__->new(shift);
+	$self = MIME::Charset->new(shift);
     }
-    my ($encoded, $charset) = $self->_text_encode($text, @_);
+    my ($encoded, $charset) = &_text_encode($self, $text, @_);
     return ($encoded, undef, 'BASE64')
 	unless $charset and $charset->{InputCharset};
     my $cset = $charset->{OutputCharset};
 
     # Determine transfer-encoding.
-    my $enc = $charset->{BodyEncoding};
+    my $enc;
+    my $dummy = $encoded;
+    eval {
+	from_to($dummy, $cset, "US-ASCII", FB_CROAK());
+    };
+    if (!$@ and $dummy eq $encoded) {
+	$cset = "US-ASCII";
+	$enc = undef;
+    } else {
+	$@ = '';
+	$enc = $charset->{BodyEncoding};
+    }
 
     if (!$enc and $encoded !~ /\x00/) {	# Eliminate hostile NUL character.
-        if ($encoded =~ $NON7BITRE) {	# String contains 8bit char(s).
+        if ($encoded =~ $NONASCIIRE) {	# String contains 8bit char(s).
             $enc = '8BIT';
-	} elsif ($cset =~ /^($ISO2022RE|$ASCIITRANSRE)$/) {	# 7BIT.
+	} elsif ($cset =~ $ISO2022RE) {	# ISO-2022-* outputs are 7BIT.
             $enc = '7BIT';
         } else {			# Pure ASCII.
             $enc = '7BIT';
             $cset = 'US-ASCII';
         }
-    } elsif ($enc eq 'S') {
-	$enc = _resolve_S($encoded, 1);
     } elsif ($enc eq 'B') {
         $enc = 'BASE64';
     } elsif ($enc eq 'Q') {
@@ -659,91 +580,6 @@ sub body_encode {
     return ($encoded, $cset, $enc);
 }
 
-=item $charset->decode(STRING [,CHECK])
-
-Decode STRING to Unicode.
-
-B<Note>:
-When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
-this function will die.
-
-=cut
-
-sub decode($$$;) {
-    my $self = shift;
-    my $s = shift;
-    my $check = shift || 0;
-    $self->{Decoder}->decode($s, $check);
-}
-
-=item detect_7bit_charset STRING
-
-Guess 7-bit charset that may encode a string STRING.
-If STRING contains any 8-bit bytes, C<undef> will be returned.
-Otherwise, Default Charset will be returned for unknown charset.
-
-=cut
-
-sub detect_7bit_charset($) {
-    return $DEFAULT_CHARSET unless &USE_ENCODE;
-    my $s = shift;
-    return $DEFAULT_CHARSET unless $s;
-
-    # Non-7bit string
-    return undef if $s =~ $NON7BITRE;
-
-    # Try to detect 7-bit escape sequences.
-    foreach (@ESCAPE_SEQS) {
-	my ($seq, $cset) = @$_;
-	if (index($s, $seq) >= 0) {
-            my $decoder = __PACKAGE__->new($cset);
-            next unless $decoder->{Decoder};
-            eval {
-		my $dummy = $s;
-		$decoder->decode($dummy, FB_CROAK());
-	    };
-	    if ($@) {
-		next;
-	    }
-	    return $decoder->{InputCharset};
-	}
-    }
-
-    # How about HZ, VIQR, UTF-7, ...?
-
-    return $DEFAULT_CHARSET;
-}
-
-sub _detect_7bit_charset {
-    detect_7bit_charset(@_);
-}
-
-=item $charset->encode(STRING [, CHECK])
-
-Encode STRING (Unicode or non-Unicode) using compatible charset recommended
-to be used for messages on Internet (if this module knows it).
-Note that string will be decoded to Unicode then encoded even if compatible charset
-was equal to original charset.
-
-B<Note>:
-When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
-this function will die.
-
-=cut
-
-sub encode($$$;) {
-    my $self = shift;
-    my $s = shift;
-    my $check = shift || 0;
-
-    unless (is_utf8($s) or $s =~ /[^\x00-\xFF]/) {
-	$s = $self->{Decoder}->decode($s, ($check & 0x1)? FB_CROAK(): 0);
-    }
-    my $enc = $self->{Encoder}->encode($s, $check);
-    Encode::_utf8_off($enc) if is_utf8($enc); # workaround for RT #35120
-    $enc;
-}
-
 =item $charset->encoded_header_len(STRING [, ENCODING])
 
 =item encoded_header_len STRING, ENCODING, CHARSET
@@ -761,12 +597,12 @@ sub encoded_header_len($$$;) {
     my ($encoding, $s);
     if (ref $self) {
 	$s = shift;
-	$encoding = uc(shift || $self->{HeaderEncoding});
+	$encoding = uc(shift) || $self->{HeaderEncoding};
     } else {
 	$s = $self;
 	$encoding = uc(shift);
 	$self  = shift;
-	$self = __PACKAGE__->new($self) unless ref $self;
+	$self = MIME::Charset->new($self) unless ref $self;
     }
 
     #FIXME:$encoding === undef
@@ -774,8 +610,9 @@ sub encoded_header_len($$$;) {
     my $enclen;
     if ($encoding eq 'Q') {
         $enclen = _enclen_Q($s);
-    } elsif ($encoding eq 'S' and _resolve_S($s) eq 'Q') {
-	$enclen = _enclen_Q($s);
+    } elsif ($encoding eq "S") {
+        my ($b, $q) = (_enclen_B($s), _enclen_Q($s));
+	$enclen = ($b < $q)? $b: $q;
     } else { # "B"
         $enclen = _enclen_B($s);
     }
@@ -787,31 +624,13 @@ sub _enclen_B($) {
     int((length(shift) + 2) / 3) * 4;
 }
 
-sub _enclen_Q($;$) {
+sub _enclen_Q($) {
     my $s = shift;
-    my $in_body = shift;
     my @o;
-    if ($in_body) {
-	@o = ($s =~ m{([^-\t\r\n !*+/0-9A-Za-z])}go);
-    } else {
-	@o = ($s =~ m{([^- !*+/0-9A-Za-z])}gos);
-    }
+    @o = ($s =~ /(\?|=|_|[^ \x21-\x7e])/gos);
     length($s) + scalar(@o) * 2;
 }
 
-sub _resolve_S($) {
-    my $s = shift;
-    my $in_body = shift;
-    my $e;
-    if ($in_body) {
-	$e = scalar(() = $s =~ m{[^-\t\r\n !*+/0-9A-Za-z]}g);
-	return (length($s) + 8 < $e * 6) ? 'BASE64' : 'QUOTED-PRINTABLE';
-    } else {
-	$e = scalar(() = $s =~ m{[^- !*+/0-9A-Za-z]}g);
-	return (length($s) + 8 < $e * 6) ? 'B' : 'Q';
-    }
-}
-
 =item $charset->header_encode(STRING [, OPTS])
 
 =item header_encode STRING, CHARSET [, OPTS]
@@ -827,15 +646,15 @@ conversion will not be performed.  So these options do not have any effects.
 
 =over 4
 
+=item Replacement => REPLACEMENT
+
+Specifies error handling scheme.  See L<"ERROR HANDLING">.
+
 =item Detect7bit => YESNO
 
 Try auto-detecting 7-bit charset when CHARSET is not given.
 Default is C<"YES">.
 
-=item Replacement => REPLACEMENT
-
-Specifies error handling scheme.  See L<"Error Handling">.
-
 =back
 
 3-item list of (I<converted string>, I<charset for output>,
@@ -848,6 +667,8 @@ data) and I<encoding scheme> will be C<undef> (should not be encoded).
 I<Charset for output> will be C<"US-ASCII"> if and only if string does not
 contain any non-ASCII bytes.
 
+=back
+
 =cut
 
 sub header_encode {
@@ -857,22 +678,37 @@ sub header_encode {
 	$text = shift;
     } else {
 	$text = $self;
-	$self = __PACKAGE__->new(shift);
+	$self = MIME::Charset->new(shift);
     }
-    my ($encoded, $charset) = $self->_text_encode($text, @_);
+    my ($encoded, $charset) = &_text_encode($self, $text, @_);
     return ($encoded, '8BIT', undef)
 	unless $charset and $charset->{InputCharset};
     my $cset = $charset->{OutputCharset};
 
     # Determine encoding scheme.
-    my $enc = $charset->{HeaderEncoding};
+    my $enc;
+    my $dummy = $encoded;
+    eval {
+	from_to($dummy, $cset, "US-ASCII", FB_CROAK());
+    };
+    if (!$@ and $dummy eq $encoded) {
+	$cset = "US-ASCII";
+	$enc = undef;
+    } else {
+	$@ = '';
+	$enc = $charset->{HeaderEncoding};
+    }
 
-    if (!$enc and $encoded !~ $NON7BITRE) {
-	unless ($cset =~ /^($ISO2022RE|$ASCIITRANSRE)$/) {	# 7BIT.
+    if (!$enc and $encoded !~ $NONASCIIRE) {
+	unless ($cset =~ $ISO2022RE) {	# ISO-2022-* outputs are 7BIT.
             $cset = 'US-ASCII';
         }
     } elsif ($enc eq 'S') {
-	$enc = _resolve_S($encoded);
+	if (_enclen_B($encoded) < _enclen_Q($encoded)) {
+	    $enc = 'B';
+	} else {
+	    $enc = 'Q';
+	}
     } elsif ($enc !~ /^[BQ]$/) {
         $enc = 'B';
     }
@@ -883,126 +719,129 @@ sub _text_encode {
     my $charset = shift;
     my $s = shift;
     my %params = @_;
-    my $replacement = uc($params{'Replacement'} || $Config->{Replacement});
-    my $detect7bit = uc($params{'Detect7bit'} || $Config->{Detect7bit});
-    my $encoding = $params{'Encoding'} ||
-	(exists $params{'Encoding'}? undef: 'A'); # undocumented
+    my $replacement = uc($params{'Replacement'}) || $Config->{Replacement};
+    my $detect7bit = uc($params{'Detect7bit'}) || $Config->{Detect7bit};
 
-    if (!$encoding or $encoding ne 'A') { # no 7-bit auto-detection
-	$detect7bit = 'NO';
-    }
-    unless ($charset->{InputCharset}) {
-	if ($s =~ $NON7BITRE) {
+    unless ($charset and $charset->{InputCharset}) {
+	if ($s =~ $NONASCIIRE) {
 	    return ($s, undef);
 	} elsif ($detect7bit ne "NO") {
-	    $charset = __PACKAGE__->new(&detect_7bit_charset($s));
+	    $charset = MIME::Charset->new(&_detect_7bit_charset($s));
 	} else {
-	    $charset = __PACKAGE__->new($DEFAULT_CHARSET,
-					Mapping => 'STANDARD');
+	    $charset = MIME::Charset->new($DEFAULT_CHARSET);
 	} 
     }
-    if (!$encoding or $encoding ne 'A') { # no conversion
-	$charset = $charset->dup;
-	$charset->encoder($charset);
-	$charset->{HeaderEncoding} = $encoding;
-	$charset->{BodyEncoding} = $encoding;
-    }
-    my $check = ($replacement and $replacement =~ /^\d+$/)?
-	$replacement:
-    {
-	'CROAK' => FB_CROAK(),
-	'STRICT' => FB_CROAK(),
-	'FALLBACK' => FB_CROAK(), # special
-	'PERLQQ' => FB_PERLQQ(),
-	'HTMLCREF' => FB_HTMLCREF(),
-	'XMLCREF' => FB_XMLCREF(),
-    }->{$replacement || ""} || 0;
+
+    # Unknown charset.
+    return ($s, $charset)
+	unless $charset->{Decoder};
 
     # Encode data by output charset if required.  If failed, fallback to
     # fallback charset.
     my $encoded;
-    if (is_utf8($s) or $s =~ /[^\x00-\xFF]/ or
-	($charset->{InputCharset} || "") ne ($charset->{OutputCharset} || "")) {
-	if ($check & 0x1) { # CROAK or FALLBACK
+
+    if (is_utf8($s) or $s =~ /[^\x00-\xFF]/) {
+	if ($replacement =~ /^(?:CROAK|STRICT|FALLBACK)$/) {
 	    eval {
 		$encoded = $s;
 		$encoded = $charset->encode($encoded, FB_CROAK());
 	    };
 	    if ($@) {
 		if ($replacement eq "FALLBACK" and $FALLBACK_CHARSET) {
-		    my $cset = __PACKAGE__->new($FALLBACK_CHARSET,
-						Mapping => 'STANDARD');
+		    $charset = MIME::Charset->new($FALLBACK_CHARSET);
 		    # croak unknown charset
-		    croak "unknown charset ``$FALLBACK_CHARSET''"
-			unless $cset->{Decoder};
-		    # charset translation
-		    $charset = $charset->dup;
-		    $charset->encoder($cset);
+		    croak "Unknown charset: $FALLBACK_CHARSET"
+			unless $charset->{Decoder};
+		    # No charset transformation.
+		    $charset->{OutputCharset} = $charset->{InputCharset};
+		    $charset->{Encoder} = $charset->{Decoder};
+
 		    $encoded = $s;
-		    $encoded = $charset->encode($encoded, 0);
-		    # replace input & output charsets with fallback charset
-		    $cset->encoder($cset);
+		    $encoded = $charset->encode($encoded);
+		} else {
+		    $@ =~ s/ at .+$//;
+		    croak $@;
+		}
+	    }
+	} elsif ($replacement eq "PERLQQ") {
+	    $encoded = $charset->encode($s, FB_PERLQQ());
+	} elsif ($replacement eq "HTMLCREF") {
+	    $encoded = $charset->encode($s, FB_HTMLCREF());
+	} elsif ($replacement eq "XMLCREF") {
+	    $encoded = $charset->encode($s, FB_XMLCREF());
+	} else {
+	    $encoded = $charset->encode($s);
+	}
+    } elsif ($charset->{InputCharset} ne $charset->{OutputCharset}) {
+	$encoded = $s;
+	if ($replacement =~ /^(?:CROAK|STRICT|FALLBACK)$/) {
+	    eval {
+		$encoded = $charset->encode($encoded, FB_CROAK());
+	    };
+	    if ($@) {
+		if ($replacement eq "FALLBACK" and $FALLBACK_CHARSET) {
+		    my $cset = MIME::Charset->new($FALLBACK_CHARSET);
+		    # croak unknown charset
+		    croak "Unknown charset: $FALLBACK_CHARSET"
+			unless $cset->{Decoder};
+		    # No charset transformations.
+		    $charset->{OutputCharset} = $cset->{OutputCharset} =
+			$cset->{InputCharset};
+		    $charset->{Encoder} = $cset->{Encoder} = $cset->{Decoder};
+		    $encoded = $s;
+		    $encoded = $charset->encode($encoded);
 		    $charset = $cset;
 		} else {
 		    $@ =~ s/ at .+$//;
 		    croak $@;
 		}
 	    }
-	} else {
-	    $encoded = $s;
-	    $encoded = $charset->encode($encoded, $check);
-	}
+        } elsif ($replacement eq "PERLQQ") {
+            $encoded = $charset->encode($encoded, FB_PERLQQ());
+        } elsif ($replacement eq "HTMLCREF") {
+            $encoded = $charset->encode($encoded, FB_HTMLCREF());
+        } elsif ($replacement eq "XMLCREF") {
+            $encoded = $charset->encode($encoded, FB_XMLCREF());
+        } else {
+            $encoded = $charset->encode($encoded);
+        }
     } else {
         $encoded = $s;
     }
 
-    if ($encoded !~ /$NONASCIIRE/) { # maybe ASCII
-	# check ``ASCII transformation'' charsets
-	if ($charset->{OutputCharset} =~ /^($ASCIITRANSRE)$/) {
-	    my $u = $encoded;
-	    if (USE_ENCODE) {
-		$u = $charset->encoder->decode($encoded); # dec. by output
-	    } elsif ($encoded =~ /[+~]/) { # workaround for pre-Encode env.
-		$u = "x$u";
-	    }
-	    if ($u eq $encoded) {
-		$charset = $charset->dup;
-		$charset->encoder($DEFAULT_CHARSET);
-	    }
-	} elsif ($charset->{OutputCharset} ne "US-ASCII") {
-	    $charset = $charset->dup;
-	    $charset->encoder($DEFAULT_CHARSET);
-	}
-    }
-
     return ($encoded, $charset);
 }
 
-=item $charset->undecode(STRING [,CHECK])
-
-Encode Unicode string STRING to byte string by input charset of $charset.
-This is equivalent to C<$charset-E<gt>decoder-E<gt>encode()>.
-
-B<Note>:
-When Unicode/multibyte support is disabled (see L<"USE_ENCODE">),
-this function will die.
-
-=cut
-
-sub undecode($$$;) {
-    my $self = shift;
+sub _detect_7bit_charset($) {
+    return $DEFAULT_CHARSET unless USE_ENCODE;
     my $s = shift;
-    my $check = shift || 0;
-    my $enc = $self->{Decoder}->encode($s, $check);
-    Encode::_utf8_off($enc); # workaround for RT #35120
-    $enc;
-}
+    return $DEFAULT_CHARSET unless $s;
 
-=back
+    # Try to detect ISO-2022-* escape sequences.
+    foreach (@ISO2022_SEQ) {
+	my ($seq, $cset) = @$_;
+	if (index($s, $seq) >= 0) {
+            my $decoder = MIME::Charset->new($cset);
+            next unless $decoder and $decoder->{Decoder};
+            eval {
+		my $dummy = $s;
+		$decoder->decode($dummy, FB_CROAK());
+	    };
+	    if ($@) {
+		next;
+	    }
+	    return $decoder->{InputCharset};
+	}
+    }
+
+    # How about HZ, VIQR, ...?
+
+    return $DEFAULT_CHARSET;
+}
 
 =head2 Manipulating Module Defaults
 
-=over
+=over 4
 
 =item alias ALIAS [, CHARSET]
 
@@ -1068,7 +907,7 @@ Otherwise, fallback charset won't be changed.  In any cases,
 current fallback charset will be returned.
 
 B<NOTE>: It I<is> useful that C<"US-ASCII"> is specified as fallback charset,
-since result of conversion will be readable without charset information.
+since result of conversion will be readable without charset informations.
 
 =cut
 
@@ -1099,8 +938,7 @@ It may be one of C<"B">, C<"Q">, C<"S"> (shorter one of either) or
 C<undef> (might not be encoded).
 
 BODYENC is recommended transfer-encoding for message body.  It may be
-one of C<"B">, C<"Q">, C<"S"> (shorter one of either) or
-C<undef> (might not be transfer-encoded).
+one of C<"B">, C<"Q"> or C<undef> (might not be transfer-encoded).
 
 ENCCHARSET is a charset which is compatible with given CHARSET and
 is recommended to be used for MIME messages on Internet.
@@ -1131,34 +969,26 @@ sub recommended ($;$;$;$) {
 	$CHARSETS{$charset} = \@spec;
 	return @spec;
     } else {
-	$charset = __PACKAGE__->new($charset) unless ref $charset;
+	$charset = MIME::Charset->new($charset) unless ref $charset;
 	return map { $charset->{$_} } qw(HeaderEncoding BodyEncoding
 					 OutputCharset);
     }
 }
 
-=back
-
 =head2 Constants
 
-=over
-
 =item USE_ENCODE
 
 Unicode/multibyte support flag.
 Non-empty string will be set when Unicode and multibyte support is enabled.
-Currently, this flag will be non-empty on Perl 5.7.3 or later and
+Currently, this flag will be non-empty on Perl 5.8.1 or later and
 empty string on earlier versions of Perl.
 
-=back
-
 =head2 Error Handling
 
 L<"body_encode"> and L<"header_encode"> accept following C<Replacement>
 options:
 
-=over
-
 =item C<"DEFAULT">
 
 Put a substitution character in place of a malformed character.
@@ -1186,11 +1016,6 @@ Synonym is C<"STRICT">.
 Use C<FB_PERLQQ>, C<FB_HTMLCREF> or C<FB_XMLCREF>
 scheme defined by L<Encode> module.
 
-=item numeric values
-
-Numeric values are also allowed.
-For more details see L<Encode/Handling Malformed Data>.
-
 =back
 
 If error handling scheme is not specified or unknown scheme is specified,
@@ -1209,62 +1034,16 @@ Consult $VERSION variable.
 Development versions of this module may be found at
 L<http://hatuka.nezumi.nu/repos/MIME-Charset/>.
 
-=head2 Incompatible Changes
-
-=over 4
-
-=item Release 1.001
-
-=over 4
-
-=item *
-
-new() method returns an object when CHARSET argument is not specified.
-
-=back
-
-=item Release 1.005
-
-=over 4
-
-=item *
-
-Restrict characters in encoded-word according to RFC 2047 section 5 (3).
-This also affects return value of encoded_header_len() method.
-
-=back
-
-=item Release 1.008.2
-
-=over 4
-
-=item *
-
-body_encoding() method may also returns C<"S">.
-
-=item *
-
-Return value of body_encode() method for UTF-8 may include
-C<"QUOTED-PRINTABLE"> encoding item that in earlier versions was fixed to
-C<"BASE64">.
-
-=back
-
-=back
-
 =head1 SEE ALSO
 
 Multipurpose Internet Mail Extensions (MIME).
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>
+Copyright (C) 2006-2008 Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>.
 
-=head1 COPYRIGHT
-
-Copyright (C) 2006-2011 Hatuka*nezumi - IKEDA Soji.
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+All rights reserved.  This program is free software; you can redistribute
+it and/or modify it under the same terms as Perl itself.
 
 =cut
 
